/******************************************************************************
 Example of advanced DHCP functionality
 Most applications can simply use the GetDhcpAddress() function call to 
 automatically handle obtaining a DHCP address and configuration. If you
 need finer control, you can create a DHCP Object and manage the DHCP
 functions from within your application. This example show how you can
 create a DHCP object and use a pointer to the object to monitor and
 control DHCP services.
 
 To obtain access to private member variables in the DHCP object, you will
 need to enable debugging in the \nburn\system\dhcpc.cpp file by removing
 the comments from the following line:
 
        #define DHCP_DEBUG 1

 You will then need to recompile the system library with a "rebuild all" from
 the IDE, or a "make" in the \nburn\system directory if you are using command
 line tools.
       
 This example will:
 - Create a DHCP object global variable
 - Start DHCP and monitor a semaphore for completion
 - Display all the DHCP information
 - Enter a while loop in which all DHCP information is displayed every
   5 seconds. 
    
*****************************************************************************/


#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <netinterface.h>


extern "C" { 
void UserMain(void * pd);
}

// Reference pointer to name for DHCP and DNS 
extern const char *pDHCPOfferName;    

// Application name that will appear in IPSetup
const char * AppName="DhcpAdvanced";

// Since we want to be able to start and stop DHCP, we need to create out
// own DHCP Object instead of using the GetDHCPAdderss() function which
// handles this automatically.
DhcpObject *pDhcpObj;

// Variable to indicate if appliction tried to obtain runtime IP settings
// from a DHCP server.
bool AssignedDHCP = FALSE;


/*-------------------------------------------------------------------
 MyStartDHCP
 Most applications can just use the GetDHCPAddress() function at boot
 time to enable the DHCP Client. If you require more control over the 
 DHCP service, such as starting and stopping, then a few lines of 
 code are required to create a DhcpObject and check a semaphore to
 determine if the DHCP request was successful. This is essentially
 what the GetDHCPAddress() function does.
 -------------------------------------------------------------------*/
void MyStartDHCP( int Interface )
{
   InterfaceBlock *ib = GetInterFaceBlock( Interface );  // Get interface data

   // The following lines of code are essentially what the GetDHCPAddress() function
   // does to make the DHCP process easier.
   pDhcpObj = new  DhcpObject( Interface ); 
   pDhcpObj->StartDHCP();    // Start DHCP

   // Pend on semaphore to verify an address was obtained
   if ( OSSemPend( &( pDhcpObj->NotifySem ), 10 * TICKS_PER_SECOND ) == OS_TIMEOUT ) //Wait 10 seconds
   {
      iprintf("\r\n\r\n*** WARNING ***\r\n");
      iprintf("IP Address was set to 0.0.0.0, and a DHCP server could not be found.\r\n");
      iprintf("Device does not have a valid IP address.\r\n\r\n");
   }
   else
   {
      iprintf("DHCP assigned the following values:\r\n");
      iprintf("IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
      iprintf("Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
      iprintf("Gate: "); ShowIP(ib->netIpGate); iprintf("\r\n");
      iprintf("DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
      iprintf("Interface Name: %s\r\n", ib->InterfaceName);
      iprintf("\r\n");
      AssignedDHCP = TRUE;
  }
}


/*-------------------------------------------------------------------
 Display runtime IP values
 This function demonstrates two methods to read the runtime IP 
 values: 
 1. Using the read-only Interface function calls
 2. Using the GetInterfaceBlock() function to get a pointer to an 
    Interface Record.
 -------------------------------------------------------------------*/
void DisplayRuntimeIPSettings(int InterfaceNumber)
{
   iprintf("\r\n\r\n--- RUNTIME IP SETTINGS ---\r\n");
   if ( AssignedDHCP )
      iprintf("Values assigned by DHCP Server\r\n");

   // Display current runtime values using Interface read-only functions
   iprintf("IP runtime settings using Interface Functions for interface %d:\r\n", 
            InterfaceNumber);
   iprintf("IP:   "); ShowIP(InterfaceIP(InterfaceNumber)); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(InterfaceMASK(InterfaceNumber)); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(InterfaceGate(InterfaceNumber)); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(InterfaceDNS(InterfaceNumber)); iprintf("\r\n");

   // Display current runtime values by getting a pointer to the Interface Block
   // and accessing it's variables.
   iprintf("\r\nIP runtime settings using GetInterfaceBlock() for interface %d:\r\n", 
           InterfaceNumber);
   InterfaceBlock *ib = GetInterFaceBlock(InterfaceNumber);
   iprintf("IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(ib->netIpGate); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
   iprintf("Interface Name: %s\r\n", ib->InterfaceName);
   iprintf("\r\n");
}


extern void PrintDhcpStatus( DhcpObject &dob );

/*-------------------------------------------------------------------
 UserMain task
 -------------------------------------------------------------------*/
void UserMain(void * pd)
{
    InitializeStack();

    pDHCPOfferName = "NetBurner";   // Host name for DNS

    int EthernetInterface = GetFirstInterface();  // Get first interface identifier
    InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
  
    if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
    {
       iprintf("\r\nNo static IP address set, attempting DHCP\r\n");
       MyStartDHCP( EthernetInterface );
    }
    else
    {
       iprintf("Static IP Address: ");ShowIP(ib->netIP);iprintf("/r/n");
       iprintf("This program can only be run with DHCP.\r\n");
       iprintf("Please set the device IP address to 0.0.0.0 and reboot\r\n");
       while(1);
    }

    OSChangePrio(MAIN_PRIO);
    EnableAutoUpdate();
    StartHTTP();
    
    iprintf("Application started");
    while (1)
    {
       PrintDhcpStatus( (DhcpObject &) *pDhcpObj );

       if ( pDhcpObj->ValidDhcpLease() )
          iprintf("ValidDhcpLease() = TRUE\r\n");
       else
          iprintf("ValidDhcpLease() = FALSE\r\n");
          
       iprintf( "GetRemainingDhcpLeaseTime() = %ld\r\n", pDhcpObj->GetRemainingDhcpLeaseTime());


       OSTimeDly( TICKS_PER_SECOND * 5 );
    }
}



